package java.util.concurrent;

import java.lang.ref.WeakReference;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.RandomAccess;
import java.util.Collection;
import java.lang.ref.Reference;
import java.lang.reflect.Constructor;
import sun.misc.Unsafe;
import java.lang.ref.ReferenceQueue;
import java.util.concurrent.locks.ReentrantLock;
import java.io.Serializable;

public abstract class ForkJoinTask<V> implements Future<V>, Serializable
{
    volatile int status;
    static final int DONE_MASK = -268435456;
    static final int NORMAL = -268435456;
    static final int CANCELLED = -1073741824;
    static final int EXCEPTIONAL = Integer.MIN_VALUE;
    static final int SIGNAL = 65536;
    static final int SMASK = 65535;
    private static final ForkJoinTask.ExceptionNode[] exceptionTable;
    private static final ReentrantLock exceptionTableLock;
    private static final ReferenceQueue<Object> exceptionTableRefQueue;
    private static final int EXCEPTION_MAP_CAPACITY = 32;
    private static final long serialVersionUID = -7721805057305804111L;
    private static final Unsafe U;
    private static final long STATUS;
    
    public ForkJoinTask() {
    }
    
    private int setCompletion(int n) {
        int status;
        while ((status = this.status) >= 0) {
            if (!ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, status, status | n)) {
                continue;
            }
            if (status >>> 16 != 0) {
                synchronized (this) {
                    this.notifyAll();
                }
            }
            return n;
        }
        return status;
    }
    
    final int doExec() {
        int n;
        if ((n = this.status) >= 0) {
            int exec;
            try {
                exec = this.exec();
            }
            catch (Throwable exceptionalCompletion) {
                return this.setExceptionalCompletion(exceptionalCompletion);
            }
            if (!(exec == 0)) {
                n = this.setCompletion(-268435456);
            }
        }
        return n;
    }
    
    final void internalWait(long timeoutMillis) {
        int status;
        if ((status = this.status) >= 0 && ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, status, status | 0x10000)) {
            synchronized (this) {
                if (this.status >= 0) {
                    try {
                        this.wait(timeoutMillis);
                    }
                    catch (InterruptedException ex) {}
                }
                else {
                    this.notifyAll();
                }
            }
        }
    }
    
    private int externalAwaitDone() {
        int expected = (this instanceof CountedCompleter) ? ForkJoinPool.common.externalHelpComplete((CountedCompleter)this, 0) : (ForkJoinPool.common.tryExternalUnpush(this) ? this.doExec() : 0);
        if (expected >= 0 && (expected = this.status) >= 0) {
            boolean n = false;
            do {
                if (ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, expected, expected | 0x10000)) {
                    synchronized (this) {
                        if (this.status >= 0) {
                            try {
                                this.wait(0L);
                            }
                            catch (InterruptedException ex) {
                                n = true;
                            }
                        }
                        else {
                            this.notifyAll();
                        }
                    }
                }
                else {
                    continue;
                }
            } while ((expected = this.status) >= 0);
            if (!(n == 0)) {
                Thread.currentThread().interrupt();
            }
        }
        return expected;
    }
    
    private int externalInterruptibleAwaitDone() throws InterruptedException {
        if (Thread.interrupted()) {
            throw new InterruptedException();
        }
        int expected;
        if ((expected = this.status) >= 0) {
            int n2;
            int n = (this instanceof CountedCompleter) ? (n2 = ForkJoinPool.common.externalHelpComplete((CountedCompleter)this, 0)) : (ForkJoinPool.common.tryExternalUnpush(this) ? (n2 = this.doExec()) : (n2 = 0));
            expected = n2;
            if (n >= 0) {
                while ((expected = this.status) >= 0) {
                    if (!ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, expected, expected | 0x10000)) {
                        continue;
                    }
                    synchronized (this) {
                        if (this.status >= 0) {
                            this.wait(0L);
                        }
                        else {
                            this.notifyAll();
                        }
                    }
                }
            }
        }
        return expected;
    }
    
    private int doJoin() {
        int status;
        Thread currentThread;
        ForkJoinWorkerThread forkJoinWorkerThread;
        ForkJoinPool.WorkQueue workQueue;
        int doExec;
        return ((status = this.status) < 0) ? status : (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread) ? (((workQueue = (forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread).workQueue).tryUnpush(this) && (doExec = this.doExec()) < 0) ? doExec : forkJoinWorkerThread.pool.awaitJoin(workQueue, this, 0L)) : this.externalAwaitDone());
    }
    
    private int doInvoke() {
        int doExec;
        int n;
        if ((doExec = this.doExec()) < 0) {
            n = doExec;
        }
        else {
            Thread currentThread;
            if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
                ForkJoinWorkerThread forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread;
                n = forkJoinWorkerThread.pool.awaitJoin(forkJoinWorkerThread.workQueue, this, 0L);
            }
            else {
                n = this.externalAwaitDone();
            }
        }
        return n;
    }
    
    final int recordExceptionalCompletion(Throwable t) {
        int n;
        if ((n = this.status) >= 0) {
            int identityHashCode = System.identityHashCode(this);
            ReentrantLock exceptionTableLock = ForkJoinTask.exceptionTableLock;
            exceptionTableLock.lock();
            Label_0115: {
                try {
                    expungeStaleExceptions();
                    ForkJoinTask.ExceptionNode[] exceptionTable = ForkJoinTask.exceptionTable;
                    int n2 = identityHashCode & exceptionTable.length - 1;
                    ForkJoinTask.ExceptionNode next = exceptionTable[n2];
                    Label_0097: {
                        while (next != null) {
                            if (next.get() == this) {
                                break Label_0115;
                            }
                            next = next.next;
                        }
                        exceptionTable[n2] = new ForkJoinTask.ExceptionNode(this, t, exceptionTable[n2]);
                        continue;
                    }
                }
                finally {
                    exceptionTableLock.unlock();
                }
            }
            n = this.setCompletion(Integer.MIN_VALUE);
        }
        return n;
    }
    
    private int setExceptionalCompletion(Throwable t) {
        int recordExceptionalCompletion = this.recordExceptionalCompletion(t);
        if ((recordExceptionalCompletion & 0xF0000000) == Integer.MIN_VALUE) {
            this.internalPropagateException(t);
        }
        return recordExceptionalCompletion;
    }
    
    void internalPropagateException(Throwable t) {
    }
    
    static final void cancelIgnoringExceptions(ForkJoinTask<?> forkJoinTask) {
        if (forkJoinTask != null && forkJoinTask.status >= 0) {
            try {
                forkJoinTask.cancel(false);
            }
            catch (Throwable t) {}
        }
    }
    
    private void clearExceptionalCompletion() {
        int identityHashCode = System.identityHashCode(this);
        ReentrantLock exceptionTableLock = ForkJoinTask.exceptionTableLock;
        exceptionTableLock.lock();
        try {
            ForkJoinTask.ExceptionNode[] exceptionTable = ForkJoinTask.exceptionTable;
            int n = identityHashCode & exceptionTable.length - 1;
            ForkJoinTask.ExceptionNode exceptionNode = exceptionTable[n];
            ForkJoinTask.ExceptionNode exceptionNode2 = null;
            while (exceptionNode != null) {
                ForkJoinTask.ExceptionNode next = exceptionNode.next;
                if (exceptionNode.get() == this) {
                    if (exceptionNode2 == null) {
                        exceptionTable[n] = next;
                        break;
                    }
                    exceptionNode2.next = next;
                    break;
                }
                else {
                    exceptionNode2 = exceptionNode;
                    exceptionNode = next;
                }
            }
            expungeStaleExceptions();
            this.status = 0;
        }
        finally {
            exceptionTableLock.unlock();
        }
    }
    
    private Throwable getThrowableException() {
        if ((this.status & 0xF0000000) != Integer.MIN_VALUE) {
            return null;
        }
        int identityHashCode = System.identityHashCode(this);
        ReentrantLock exceptionTableLock = ForkJoinTask.exceptionTableLock;
        exceptionTableLock.lock();
        ForkJoinTask.ExceptionNode next;
        try {
            expungeStaleExceptions();
            ForkJoinTask.ExceptionNode[] exceptionTable = ForkJoinTask.exceptionTable;
            for (next = exceptionTable[identityHashCode & exceptionTable.length - 1]; next != null && next.get() != this; next = next.next) {}
        }
        finally {
            exceptionTableLock.unlock();
        }
        Throwable ex;
        if (next == null || (ex = next.ex) == null) {
            return null;
        }
        if (next.thrower != Thread.currentThread().getId()) {
            Class<Object> class1 = ex.getClass();
            try {
                Constructor<Object> constructor = null;
                Constructor[] constructors = class1.getConstructors();
                for (int i = 0; i < constructors.length; ++i) {
                    Constructor<Object> constructor2 = constructors[i];
                    Class[] parameterTypes = constructor2.getParameterTypes();
                    if (parameterTypes.length == 0) {
                        constructor = constructor2;
                    }
                    else if (parameterTypes.length == 1 && parameterTypes[0] == Throwable.class) {
                        Throwable t = (Throwable)constructor2.newInstance(new Object[] { ex });
                        return (t == null) ? ex : t;
                    }
                }
                if (constructor != null) {
                    Throwable t2 = (Throwable)(Throwable)constructor.newInstance(new Object[0]);
                    if (t2 != null) {
                        t2.initCause(ex);
                        return t2;
                    }
                }
            }
            catch (Exception ex2) {}
        }
        return ex;
    }
    
    private static void expungeStaleExceptions() {
        Reference<Object> poll;
        while ((poll = ForkJoinTask.exceptionTableRefQueue.poll()) != null) {
            if (!(poll instanceof ForkJoinTask.ExceptionNode)) {
                continue;
            }
            int hashCode = ((ForkJoinTask.ExceptionNode)poll).hashCode;
            ForkJoinTask.ExceptionNode[] exceptionTable = ForkJoinTask.exceptionTable;
            int n = hashCode & exceptionTable.length - 1;
            ForkJoinTask.ExceptionNode exceptionNode = exceptionTable[n];
            ForkJoinTask.ExceptionNode exceptionNode2 = null;
            while (exceptionNode != null) {
                ForkJoinTask.ExceptionNode next = exceptionNode.next;
                if (exceptionNode == poll) {
                    if (exceptionNode2 == null) {
                        exceptionTable[n] = next;
                        break;
                    }
                    exceptionNode2.next = next;
                    break;
                }
                else {
                    exceptionNode2 = exceptionNode;
                    exceptionNode = next;
                }
            }
        }
    }
    
    static final void helpExpungeStaleExceptions() {
        ReentrantLock exceptionTableLock = ForkJoinTask.exceptionTableLock;
        if (exceptionTableLock.tryLock()) {
            try {
                expungeStaleExceptions();
            }
            finally {
                exceptionTableLock.unlock();
            }
        }
    }
    
    static void rethrow(Throwable t) {
        if (t != null) {
            uncheckedThrow(t);
        }
    }
    
    static <T extends Throwable> void uncheckedThrow(Throwable t) throws T, Throwable {
        throw t;
    }
    
    private void reportException(int n) {
        if (n == -1073741824) {
            throw new CancellationException();
        }
        if (n == Integer.MIN_VALUE) {
            rethrow(this.getThrowableException());
        }
        return;
    }
    
    public final ForkJoinTask<V> fork() {
        Thread currentThread;
        if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
            ((ForkJoinWorkerThread)currentThread).workQueue.push(this);
            return this;
        }
        ForkJoinPool.common.externalPush(this);
        return this;
    }
    
    public final V join() {
        int n;
        if ((n = (this.doJoin() & 0xF0000000)) != -268435456) {
            this.reportException(n);
        }
        return this.getRawResult();
    }
    
    public final V invoke() {
        int n;
        if ((n = (this.doInvoke() & 0xF0000000)) != -268435456) {
            this.reportException(n);
        }
        return this.getRawResult();
    }
    
    public static void invokeAll(ForkJoinTask<?> forkJoinTask, ForkJoinTask<?> forkJoinTask2) {
        forkJoinTask2.fork();
        int n;
        if ((n = (forkJoinTask.doInvoke() & 0xF0000000)) != -268435456) {
            forkJoinTask.reportException(n);
        }
        int n2;
        if ((n2 = (forkJoinTask2.doJoin() & 0xF0000000)) != -268435456) {
            forkJoinTask2.reportException(n2);
        }
    }
    
    public static void invokeAll(ForkJoinTask<?>... array) {
        Throwable t = null;
        int i;
        for (i = (i = array.length - 1); i >= 0; --i) {
            ForkJoinTask<?> forkJoinTask = array[i];
            if (forkJoinTask == null) {
                if (t == null) {
                    t = new NullPointerException();
                }
            }
            else if (i != 0) {
                forkJoinTask.fork();
            }
            else if (forkJoinTask.doInvoke() < -268435456 && t == null) {
                t = forkJoinTask.getException();
            }
        }
        for (int j = 1; j <= i; ++j) {
            ForkJoinTask<?> forkJoinTask2 = array[j];
            if (forkJoinTask2 != null) {
                if (t != null) {
                    forkJoinTask2.cancel(false);
                }
                else if (forkJoinTask2.doJoin() < -268435456) {
                    t = forkJoinTask2.getException();
                }
            }
        }
        if (t != null) {
            rethrow(t);
        }
    }
    
    public static <T extends ForkJoinTask<?>> Collection<T> invokeAll(Collection<T> collection) {
        if (!(collection instanceof RandomAccess) || !(collection instanceof List)) {
            invokeAll((ForkJoinTask[])collection.toArray(new ForkJoinTask[collection.size()]));
            return collection;
        }
        List<Object> list = (List)collection;
        Throwable t = null;
        int i;
        for (i = (i = list.size() - 1); i >= 0; --i) {
            ForkJoinTask<Object> forkJoinTask = (ForkJoinTask)list.get(i);
            if (forkJoinTask == null) {
                if (t == null) {
                    t = new NullPointerException();
                }
            }
            else if (i != 0) {
                forkJoinTask.fork();
            }
            else if (forkJoinTask.doInvoke() < -268435456 && t == null) {
                t = forkJoinTask.getException();
            }
        }
        for (int j = 1; j <= i; ++j) {
            ForkJoinTask<Object> forkJoinTask2 = (ForkJoinTask)list.get(j);
            if (forkJoinTask2 != null) {
                if (t != null) {
                    forkJoinTask2.cancel(false);
                }
                else if (forkJoinTask2.doJoin() < -268435456) {
                    t = forkJoinTask2.getException();
                }
            }
        }
        if (t != null) {
            rethrow(t);
            return collection;
        }
        return collection;
    }
    
    @Override
    public boolean cancel(boolean b) {
        return (this.setCompletion(-1073741824) & 0xF0000000) == 0xC0000000;
    }
    
    @Override
    public final boolean isDone() {
        return this.status < 0;
    }
    
    @Override
    public final boolean isCancelled() {
        return (this.status & 0xF0000000) == 0xC0000000;
    }
    
    public final boolean isCompletedAbnormally() {
        return this.status < -268435456;
    }
    
    public final boolean isCompletedNormally() {
        return (this.status & 0xF0000000) == 0xF0000000;
    }
    
    public final Throwable getException() {
        int n = this.status & 0xF0000000;
        return (n >= -268435456) ? null : ((n == -1073741824) ? new CancellationException() : this.getThrowableException());
    }
    
    public void completeExceptionally(Throwable cause) {
        this.setExceptionalCompletion((cause instanceof RuntimeException || cause instanceof Error) ? cause : new RuntimeException(cause));
    }
    
    public void complete(V rawResult) {
        try {
            this.setRawResult(rawResult);
        }
        catch (Throwable exceptionalCompletion) {
            this.setExceptionalCompletion(exceptionalCompletion);
            return;
        }
        this.setCompletion(-268435456);
    }
    
    public final void quietlyComplete() {
        this.setCompletion(-268435456);
    }
    
    @Override
    public final V get() throws InterruptedException, ExecutionException {
        int n;
        if ((n = (((Thread.currentThread() instanceof ForkJoinWorkerThread) ? this.doJoin() : this.externalInterruptibleAwaitDone()) & 0xF0000000)) == -1073741824) {
            throw new CancellationException();
        }
        Throwable throwableException;
        if (n == Integer.MIN_VALUE && (throwableException = this.getThrowableException()) != null) {
            throw new ExecutionException(throwableException);
        }
        return this.getRawResult();
    }
    
    @Override
    public final V get(long n, TimeUnit timeUnit) throws InterruptedException, ExecutionException, TimeoutException {
        long nanos = timeUnit.toNanos(n);
        if (Thread.interrupted()) {
            throw new InterruptedException();
        }
        int expected;
        if ((expected = this.status) >= 0 && nanos > 0L) {
            long n2 = System.nanoTime() + nanos;
            n2 = ((n2 == 0L) ? 1L : n2);
            Thread currentThread = Thread.currentThread();
            if ((currentThread instanceof ForkJoinWorkerThread)) {
                ForkJoinWorkerThread forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread;
                expected = forkJoinWorkerThread.pool.awaitJoin(forkJoinWorkerThread.workQueue, this, n2);
            }
            else {
                int n4;
                int n3 = (this instanceof CountedCompleter) ? (n4 = ForkJoinPool.common.externalHelpComplete((CountedCompleter)this, 0)) : (ForkJoinPool.common.tryExternalUnpush(this) ? (n4 = this.doExec()) : (n4 = 0));
                expected = n4;
                if (n3 >= 0) {
                    long n5;
                    while ((expected = this.status) >= 0 && (n5 = n2 - System.nanoTime()) > 0L) {
                        long millis;
                        if ((millis = TimeUnit.NANOSECONDS.toMillis(n5)) <= 0L || !ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, expected, expected | 0x10000)) {
                            continue;
                        }
                        synchronized (this) {
                            if (this.status >= 0) {
                                this.wait(millis);
                            }
                            else {
                                this.notifyAll();
                            }
                        }
                    }
                }
            }
        }
        if (expected >= 0) {
            expected = this.status;
        }
        int n6;
        if ((n6 = (expected & 0xF0000000)) != -268435456) {
            if (n6 == -1073741824) {
                throw new CancellationException();
            }
            if (n6 != Integer.MIN_VALUE) {
                throw new TimeoutException();
            }
            Throwable throwableException;
            if ((throwableException = this.getThrowableException()) != null) {
                throw new ExecutionException(throwableException);
            }
        }
        return this.getRawResult();
    }
    
    public final void quietlyJoin() {
        this.doJoin();
    }
    
    public final void quietlyInvoke() {
        this.doInvoke();
    }
    
    public static void helpQuiesce() {
        Thread currentThread;
        if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
            ForkJoinWorkerThread forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread;
            forkJoinWorkerThread.pool.helpQuiescePool(forkJoinWorkerThread.workQueue);
        }
        else {
            ForkJoinPool.quiesceCommonPool();
        }
    }
    
    public void reinitialize() {
        if ((this.status & 0xF0000000) == Integer.MIN_VALUE) {
            this.clearExceptionalCompletion();
        }
        else {
            this.status = 0;
        }
    }
    
    public static ForkJoinPool getPool() {
        Thread currentThread = Thread.currentThread();
        return (currentThread instanceof ForkJoinWorkerThread) ? ((ForkJoinWorkerThread)currentThread).pool : null;
    }
    
    public static boolean inForkJoinPool() {
        return Thread.currentThread() instanceof ForkJoinWorkerThread;
    }
    
    public boolean tryUnfork() {
        Thread currentThread;
        return ((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread) ? ((ForkJoinWorkerThread)currentThread).workQueue.tryUnpush(this) : ForkJoinPool.common.tryExternalUnpush(this);
    }
    
    public static int getQueuedTaskCount() {
        Thread currentThread;
        ForkJoinPool.WorkQueue workQueue;
        if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
            workQueue = ((ForkJoinWorkerThread)currentThread).workQueue;
        }
        else {
            workQueue = ForkJoinPool.commonSubmitterQueue();
        }
        return (workQueue == null) ? 0 : workQueue.queueSize();
    }
    
    public static int getSurplusQueuedTaskCount() {
        return ForkJoinPool.getSurplusQueuedTaskCount();
    }
    
    public abstract V getRawResult();
    
    protected abstract void setRawResult(V p0);
    
    protected abstract boolean exec();
    
    protected static ForkJoinTask<?> peekNextLocalTask() {
        Thread currentThread;
        ForkJoinPool.WorkQueue workQueue;
        if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
            workQueue = ((ForkJoinWorkerThread)currentThread).workQueue;
        }
        else {
            workQueue = ForkJoinPool.commonSubmitterQueue();
        }
        return (workQueue == null) ? null : workQueue.peek();
    }
    
    protected static ForkJoinTask<?> pollNextLocalTask() {
        Thread currentThread;
        return ((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread) ? ((ForkJoinWorkerThread)currentThread).workQueue.nextLocalTask() : null;
    }
    
    protected static ForkJoinTask<?> pollTask() {
        Thread currentThread;
        if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
            ForkJoinWorkerThread forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread;
            return forkJoinWorkerThread.pool.nextTaskFor(forkJoinWorkerThread.workQueue);
        }
        return null;
    }
    
    public final short getForkJoinTaskTag() {
        return (short)this.status;
    }
    
    public final short setForkJoinTaskTag(short n) {
        int status;
        while (!ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, status = this.status, (status & 0xFFFF0000) | (n & 0xFFFF))) {}
        return (short)status;
    }
    
    public final boolean compareAndSetForkJoinTaskTag(short n, short n2) {
        int status;
        while ((short)(status = this.status) == n) {
            if (!ForkJoinTask.U.compareAndSwapInt(this, ForkJoinTask.STATUS, status, (status & 0xFFFF0000) | (n2 & 0xFFFF))) {
                continue;
            }
            return true;
        }
        return false;
    }
    
    public static ForkJoinTask<?> adapt(Runnable runnable) {
        return new AdaptedRunnableAction(runnable);
    }
    
    public static <T> ForkJoinTask<T> adapt(Runnable runnable, T t) {
        return new ForkJoinTask.AdaptedRunnable(runnable, t);
    }
    
    public static <T> ForkJoinTask<T> adapt(Callable<? extends T> callable) {
        return new ForkJoinTask.AdaptedCallable(callable);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeObject(this.getException());
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        Object object = objectInputStream.readObject();
        if (object != null) {
            this.setExceptionalCompletion((Throwable)object);
        }
    }
    
    static {
        exceptionTableLock = new ReentrantLock();
        exceptionTableRefQueue = new ReferenceQueue();
        exceptionTable = new ForkJoinTask.ExceptionNode[32];
        try {
            U = Unsafe.getUnsafe();
            STATUS = ForkJoinTask.U.objectFieldOffset(ForkJoinTask.class.getDeclaredField("status"));
        }
        catch (Exception cause) {
            throw new Error(cause);
        }
    }
    
    static final class ExceptionNode extends WeakReference<ForkJoinTask<?>>
    {
        final Throwable ex;
        ForkJoinTask.ExceptionNode next;
        final long thrower;
        final int hashCode;
        
        ExceptionNode(ForkJoinTask<?> referent, Throwable ex, ForkJoinTask.ExceptionNode next) {
            super(referent, ForkJoinTask.exceptionTableRefQueue);
            this.ex = ex;
            this.next = next;
            this.thrower = Thread.currentThread().getId();
            this.hashCode = System.identityHashCode(referent);
        }
    }
    
    static final class AdaptedRunnable<T> extends ForkJoinTask<T> implements RunnableFuture<T>
    {
        final Runnable runnable;
        T result;
        private static final long serialVersionUID = 5232453952276885070L;
        
        AdaptedRunnable(Runnable runnable, T result) {
            if (runnable == null) {
                throw new NullPointerException();
            }
            this.runnable = runnable;
            this.result = result;
            return;
        }
        
        @Override
        public final T getRawResult() {
            return this.result;
        }
        
        public final void setRawResult(T result) {
            this.result = result;
        }
        
        public final boolean exec() {
            this.runnable.run();
            return true;
        }
        
        @Override
        public final void run() {
            this.invoke();
        }
    }
    
    static final class AdaptedRunnableAction extends ForkJoinTask<Void> implements RunnableFuture<Void>
    {
        final Runnable runnable;
        private static final long serialVersionUID = 5232453952276885070L;
        
        AdaptedRunnableAction(Runnable runnable) {
            if (runnable == null) {
                throw new NullPointerException();
            }
            this.runnable = runnable;
            return;
        }
        
        @Override
        public final Void getRawResult() {
            return null;
        }
        
        public final void setRawResult(Void void1) {
        }
        
        public final boolean exec() {
            this.runnable.run();
            return true;
        }
        
        @Override
        public final void run() {
            this.invoke();
        }
    }
    
    static final class RunnableExecuteAction extends ForkJoinTask<Void>
    {
        final Runnable runnable;
        private static final long serialVersionUID = 5232453952276885070L;
        
        RunnableExecuteAction(Runnable runnable) {
            if (runnable == null) {
                throw new NullPointerException();
            }
            this.runnable = runnable;
            return;
        }
        
        @Override
        public final Void getRawResult() {
            return null;
        }
        
        public final void setRawResult(Void void1) {
        }
        
        public final boolean exec() {
            this.runnable.run();
            return true;
        }
        
        @Override
        void internalPropagateException(Throwable t) {
            ForkJoinTask.rethrow(t);
        }
    }
    
    static final class AdaptedCallable<T> extends ForkJoinTask<T> implements RunnableFuture<T>
    {
        final Callable<? extends T> callable;
        T result;
        private static final long serialVersionUID = 2838392045355241008L;
        
        AdaptedCallable(Callable<? extends T> callable) {
            if (callable == null) {
                throw new NullPointerException();
            }
            this.callable = callable;
            return;
        }
        
        @Override
        public final T getRawResult() {
            return this.result;
        }
        
        public final void setRawResult(T result) {
            this.result = result;
        }
        
        public final boolean exec() {
            try {
                this.result = this.callable.call();
                return true;
            }
            catch (Error error) {
                throw error;
            }
            catch (RuntimeException ex) {
                throw ex;
            }
            catch (Exception cause) {
                throw new RuntimeException(cause);
            }
        }
        
        @Override
        public final void run() {
            this.invoke();
        }
    }
}
